/* /////////////////////////////////////////////////////////////////////////////
 * File:        recls_internal.cpp
 *
 * Purpose:     Implementation file for the recls API internal helpers.
 *
 * Created:     16th August 2003
 * Updated:     2nd November 2003
 *
 * Copyright:   Synesis Software Pty Ltd, (c) 2003. All rights reserved.
 *
 * Home:        http://www.synesis.com.au/software
 *              http://recls.org/
 *
 * ////////////////////////////////////////////////////////////////////////// */


/* /////////////////////////////////////////////////////////////////////////////
 * Includes
 */

#include "recls.h"
#include "recls_internal.h"
#include "recls_assert.h"

#include <stlsoft_nulldef.h>

#include <string.h>

/* /////////////////////////////////////////////////////////////////////////////
 * Namespace
 */

#if !defined(RECLS_NO_NAMESPACE)
namespace recls
{
#endif /* !RECLS_NO_NAMESPACE */

/* ////////////////////////////////////////////////////////////////////////// */

static size_t recls_strncpy(recls_char_t *dest, size_t cchDest, recls_char_t const *src, size_t cchSrc)
{
    size_t  cchWritten;

    if(cchDest < cchSrc)
    {
        /* Just to straight strncpy. */
        strncpy(dest, src, cchDest);

        cchWritten = cchDest;
    }
    else
    {
        strncpy(dest, src, cchSrc);

        if(cchSrc < cchDest)
        {
            /* Fill the rest up with blanks. */

            memset(&dest[cchSrc], 0, sizeof(recls_char_t) * (cchDest - cchSrc));
        }

        cchWritten = cchSrc;
    }

    return cchWritten;
}

/* ////////////////////////////////////////////////////////////////////////// */

#if defined(RECLS_COMPILER_IS_DMC) || \
    defined(RECLS_COMPILER_IS_WATCOM)
RECLS_FNDECL(size_t) Recls_GetStringProperty_(  struct recls_strptrs_t const *      ptrs
                                            ,   recls_char_t *                      buffer
                                            ,   size_t                              cchBuffer)
#else
RECLS_FNDECL(size_t) Recls_GetStringProperty_(  struct recls_strptrs_t const *const ptrs
                                            ,   recls_char_t *const                 buffer
                                            ,   size_t const                        cchBuffer)
#endif /* RECLS_COMPILER_IS_DMC || RECLS_COMPILER_IS_WATCOM */
{
    recls_assert(NULL != ptrs);

    size_t  cch =   ptrs->end - ptrs->begin;

    if(NULL != buffer)
    {
        cch = recls_strncpy(buffer, cchBuffer, ptrs->begin, cch);
    }

    return cch;
}

/* /////////////////////////////////////////////////////////////////////////////
 * Namespace
 */

#if !defined(RECLS_NO_NAMESPACE)
} /* namespace recls */
#endif /* !RECLS_NO_NAMESPACE */

/* ////////////////////////////////////////////////////////////////////////// */
