/* /////////////////////////////////////////////////////////////////////////////
 * File:        recls_util.cpp
 *
 * Purpose:     Platform-independent utility functions for the recls API.
 *
 * Created:     17th August 2003
 * Updated:     2nd November 2003
 *
 * Copyright:   Synesis Software Pty Ltd, (c) 2003. All rights reserved.
 *
 * Home:        http://www.synesis.com.au/software
 *              http://recls.org/
 *
 * ////////////////////////////////////////////////////////////////////////// */


/* /////////////////////////////////////////////////////////////////////////////
 * Includes
 */

#include "recls.h"
#include "recls_internal.h"
#include "recls_assert.h"
#include "recls_util.h"

#include <stlsoft_nulldef.h>

/* /////////////////////////////////////////////////////////////////////////////
 * Namespace
 */

#if !defined(RECLS_NO_NAMESPACE)
namespace recls
{
#endif /* !RECLS_NO_NAMESPACE */

/* ////////////////////////////////////////////////////////////////////////// */

RECLS_LINKAGE_C size_t align_up_size(size_t i)
{
    return (size_t)((i + (4 - 1)) & ~(4 - 1));
}

RECLS_LINKAGE_C recls_bool_t is_dots(recls_char_t const *f)
{
    recls_assert(NULL != f);

    return  (   f[0] == '.' &&
                f[1] == '\0') ||
            (   f[0] == '.' &&
                f[1] == '.' &&
                f[2] == '\0');
}

RECLS_LINKAGE_C size_t count_char_instances_a(recls_char_a_t const *begin, recls_char_a_t const *end, recls_char_a_t const ch)
{
    size_t  cDirParts   =   0;

    for(; begin != end; ++begin)
    {
        if(*begin == ch)
        {
            ++cDirParts;
        }
    }

    return cDirParts;
}

RECLS_LINKAGE_C size_t count_char_instances_w(recls_char_w_t const *begin, recls_char_w_t const *end, recls_char_w_t const ch)
{
    size_t  cDirParts   =   0;

    for(; begin != end; ++begin)
    {
        if(*begin == ch)
        {
            ++cDirParts;
        }
    }

    return cDirParts;
}

/* /////////////////////////////////////////////////////////////////////////////
 * Namespace
 */

#if !defined(RECLS_NO_NAMESPACE)
} /* namespace recls */
#endif /* !RECLS_NO_NAMESPACE */

/* ////////////////////////////////////////////////////////////////////////// */
