/* /////////////////////////////////////////////////////////////////////////////
 * File:        recls_util_win32.cpp
 *
 * Purpose:     Win32 utility functions for the recls API.
 *
 * Created:     17th August 2003
 * Updated:     23rd November 2003
 *
 * Copyright:   Synesis Software Pty Ltd, (c) 2003. All rights reserved.
 *
 * Home:        http://www.synesis.com.au/software
 *              http://recls.org/
 *
 * ////////////////////////////////////////////////////////////////////////// */


/* /////////////////////////////////////////////////////////////////////////////
 * Includes
 */

#include "recls.h"
#include "recls_internal.h"
#include "recls_assert.h"
#include "recls_util.h"

#include <stlsoft_nulldef.h>

#include <unixstl_filesystem_traits.h>

#include <sys/types.h>
#include <sys/stat.h>

#include <errno.h>

/* /////////////////////////////////////////////////////////////////////////////
 * Namespace
 */

#if !defined(RECLS_NO_NAMESPACE)
namespace recls
{
#endif /* !RECLS_NO_NAMESPACE */

unixstl_ns_using(filesystem_traits)

/* ////////////////////////////////////////////////////////////////////////// */

RECLS_LINKAGE_C recls_bool_t file_exists(recls_char_t const *f)
{
	struct stat	st;

	return 0 == stat(f, &st) || errno != ENOENT;
}

RECLS_LINKAGE_C size_t count_dir_parts_a(recls_char_a_t const *begin, recls_char_a_t const *end)
{
	return count_char_instances_a(begin, end, unixstl_ns_qual(filesystem_traits)<recls_char_a_t>::path_name_separator());
}

#if 0
RECLS_LINKAGE_C size_t count_dir_parts_w(recls_char_w_t const *begin, recls_char_w_t const *end)
{
	return count_char_instances_w(begin, end, unixstl_ns_qual(filesystem_traits)<recls_char_w_t>::path_name_separator());
}
#endif /* 0 */

/* /////////////////////////////////////////////////////////////////////////////
 * Namespace
 */

#if !defined(RECLS_NO_NAMESPACE)
} /* namespace recls */
#endif /* !RECLS_NO_NAMESPACE */

/* ////////////////////////////////////////////////////////////////////////// */
