
// Updated: 24th November 2003

#include "recls_jni_util.h"

#include <stlsoft_auto_buffer.h>
#include <stlsoft_malloc_allocator.h>

#include <winstl_time_functions.h>

typedef	stlsoft_ns_qual(auto_buffer)<recls_char_t, stlsoft_ns_qual(malloc_allocator)<recls_char_t>, RECLS_PATH_MAX>	buffer_t;

void ThrowException(JNIEnv *henv, char const *className, char const *message)
{
	va_list	args;

	va_start(args, message);

	jclass	clsX	=	henv->FindClass(className);

	if(NULL != clsX)
	{
		henv->ThrowNew(clsX, message);

		henv->DeleteLocalRef(clsX);
	}

	va_end(args);
}

void ThrowReclsException(JNIEnv *henv, char const *message, recls_rc_t code)
{
	jclass		clsX		=	henv->FindClass(CLASSNAME_RECLSX);
	jmethodID	midXCtor	=	henv->GetMethodID(clsX, "<init>", "(Ljava/lang/String;I)V");
	jstring		jsMessage	=	henv->NewStringUTF(message);
	jobject		x			=	henv->NewObject(clsX, midXCtor, jsMessage, code);

	henv->Throw(static_cast<jthrowable>(x));

	henv->DeleteLocalRef(clsX);
}

jstring StringFromRange(JNIEnv *henv, recls_char_t const *from, recls_char_t const *to)
{
	buffer_t	buffer(1 + (to - from));

#ifdef RECLS_CHAR_TYPE_IS_CHAR
	strncpy(buffer, from, buffer.size() - 1);
	buffer[buffer.size() - 1] = 0;

	return henv->NewStringUTF(buffer);
#elif defined(RECLS_CHAR_TYPE_IS_WCHAR
	wcsncpy(buffer, from, buffer.size() - 1);
	buffer[buffer.size() - 1] = 0;

	return henv->NewString(buffer);
#else /* ?char type */
# error recls char type must be discriminated
#endif /* char type */
}

jsize SafeStringUTFLength(JNIEnv *henv, jstring str)
{
	return NULL == str ? 0 : henv->GetStringUTFLength(str);
}

jsize SafeStringLength(JNIEnv *henv, jstring str)
{
	return NULL == str ? 0 : henv->GetStringLength(str);
}

char const *SafeStringUTFChars(JNIEnv *henv, jstring str, jboolean *isCopy)
{
	return NULL == str ? "" : henv->GetStringUTFChars(str, isCopy);
}

jchar const *SafeStringChars(JNIEnv *henv, jstring str, jboolean *isCopy)
{
	return NULL == str ? L"" : henv->GetStringChars(str, isCopy);
}
