/* /////////////////////////////////////////////////////////////////////////////
 * File:        ReclsException.java
 *
 * Purpose:     Implementation of the org.recls.ReclsException class
 *
 * Created      21st November 2003
 * Updated:     24th November 2003
 *
 * Author:      Matthew Wilson
 *
 * License:     (Licensed under the Synesis Software Standard Source License)
 *
 *              Copyright (C) 2002-2003, Synesis Software Pty Ltd.
 *
 *              All rights reserved.
 *
 *              www:        http://www.recls.org/
 *                          http://www.synesis.com.au/software
 *                          http://www.synsoft.org/
 *
 *              email:      admin@recls.org
 *
 *              Redistribution and use in source and binary forms, with or
 *              without modification, are permitted provided that the following
 *              conditions are met:
 *
 *              (i) Redistributions of source code must retain the above
 *              copyright notice and contact information, this list of
 *              conditions and the following disclaimer.
 *
 *              (ii) Any derived versions of this software (howsoever modified)
 *              remain the sole property of Synesis Software.
 *
 *              (iii) Any derived versions of this software (howsoever modified)
 *              remain subject to all these conditions.
 *
 *              (iv) Neither the name of Synesis Software nor the names of any
 *              subdivisions, employees or agents of Synesis Software, nor the
 *              names of any other contributors to this software may be used to
 *              endorse or promote products derived from this software without
 *              specific prior written permission.
 *
 *              This source code is provided by Synesis Software "as is" and any
 *              warranties, whether expressed or implied, including, but not
 *              limited to, the implied warranties of merchantability and
 *              fitness for a particular purpose are disclaimed. In no event
 *              shall the Synesis Software be liable for any direct, indirect,
 *              incidental, special, exemplary, or consequential damages
 *              (including, but not limited to, procurement of substitute goods
 *              or services; loss of use, data, or profits; or business
 *              interruption) however caused and on any theory of liability,
 *              whether in contract, strict liability, or tort (including
 *              negligence or otherwise) arising in any way out of the use of
 *              this software, even if advised of the possibility of such
 *              damage.
 *
 * ////////////////////////////////////////////////////////////////////////// */


package org.recls;

import java.lang.Exception;

/** This class is thrown to represent recls-specific exceptional conditions
 */
public class ReclsException
    extends Exception
{
    /** Constructs an exception from the given message and error code
     */
    public ReclsException(String message, int code)
    {
        super(message);

        m_code = code;
    }

    /** Returns the value of the error code associated with the exception
     */
    public int getCode()
    {
        return m_code;
    }

    /** Translates a given error code to a human-readable string
     */
    public static native String translateCode(int code);

    /** Translates the assocated error code to a human-readable string
     */
    public String translateCode()
    {
        return translateCode(m_code);
    }

/// Members
    private int     m_code;
}
