/* /////////////////////////////////////////////////////////////////////////////
 * File:        Search.java
 *
 * Purpose:     Implementation of the org.recls.Search class
 *
 * Created      21st November 2003
 * Updated:     24th November 2003
 *
 * Author:      Matthew Wilson
 *
 * License:     (Licensed under the Synesis Software Standard Source License)
 *
 *              Copyright (C) 2002-2003, Synesis Software Pty Ltd.
 *
 *              All rights reserved.
 *
 *              www:        http://www.recls.org/
 *                          http://www.synesis.com.au/software
 *                          http://www.synsoft.org/
 *
 *              email:      admin@recls.org
 *
 *              Redistribution and use in source and binary forms, with or
 *              without modification, are permitted provided that the following
 *              conditions are met:
 *
 *              (i) Redistributions of source code must retain the above
 *              copyright notice and contact information, this list of
 *              conditions and the following disclaimer.
 *
 *              (ii) Any derived versions of this software (howsoever modified)
 *              remain the sole property of Synesis Software.
 *
 *              (iii) Any derived versions of this software (howsoever modified)
 *              remain subject to all these conditions.
 *
 *              (iv) Neither the name of Synesis Software nor the names of any
 *              subdivisions, employees or agents of Synesis Software, nor the
 *              names of any other contributors to this software may be used to
 *              endorse or promote products derived from this software without
 *              specific prior written permission.
 *
 *              This source code is provided by Synesis Software "as is" and any
 *              warranties, whether expressed or implied, including, but not
 *              limited to, the implied warranties of merchantability and
 *              fitness for a particular purpose are disclaimed. In no event
 *              shall the Synesis Software be liable for any direct, indirect,
 *              incidental, special, exemplary, or consequential damages
 *              (including, but not limited to, procurement of substitute goods
 *              or services; loss of use, data, or profits; or business
 *              interruption) however caused and on any theory of liability,
 *              whether in contract, strict liability, or tort (including
 *              negligence or otherwise) arising in any way out of the use of
 *              this software, even if advised of the possibility of such
 *              damage.
 *
 * ////////////////////////////////////////////////////////////////////////// */


package org.recls;

import java.io.File;
import java.util.Enumeration;

/** This class represents a recursive search
 */
public class Search
    implements Enumeration
{
    public static final int RECLS_F_FILES               =   0x00000001; /*!< Include files in search. Included by default if none specified */
    public static final int RECLS_F_DIRECTORIES         =   0x00000002; /*!< Include directories in search. Not currently supported. */
    public static final int RECLS_F_LINKS               =   0x00000004; /*!< Include links in search. Ignored in Win32. */
    public static final int RECLS_F_DEVICES             =   0x00000008; /*!< Include devices in search. Not currently supported. */
//  public static final int RECLS_F_TYPEMASK            =   0x00000FFF;
    public static final int RECLS_F_RECURSIVE           =   0x00010000; /*!< Searches given directory and all sub-directories */
    public static final int RECLS_F_NO_FOLLOW_LINKS     =   0x00020000; /*!< Does not expand links */
    public static final int RECLS_F_DIRECTORY_PARTS     =   0x00040000; /*!< Fills out the directory parts. Supported from version 1.1.1 onwards. */
    public static final int RECLS_F_DETAILS_LATER       =   0x00080000; /*!< Does not fill out anything other than the path. Not currently supported. */


    private Search(String searchRoot, String pattern, int flags)
    {
        // Ctors cannot be native
        m_hSearch   =   Initialise(searchRoot, pattern, flags);
    }

    /** Releases the resources owned by the search
     */
    public native void Close();
    /** Releases the resources owned by the search
     */
    public void Finalize()
    {
        Close();
    }

    /** Creates a new enumerable search */
    public static Search MakeSearch(String searchRoot, String pattern, int flags) throws ReclsException
    {
        return new Search(searchRoot, pattern, flags);
    }

    /** Tests if this enumeration contains more elements
     */
    public final boolean hasMoreElements()
    {
        return hasMoreElements_();
    }
        
    /** Returns the next element of this enumeration if this enumeration object has at least one more element to provide. 
     *
     * \note This throws java.lang.NoSuchElementException if hasMoreElements() would return false.
     */
    public final Object nextElement()
    {
        return nextElement_();
    }

/// Native method implementations
    private native int      Initialise(String searchRoot, String pattern, int flags);
    private native boolean  hasMoreElements_();
    private native Object   nextElement_();

// Implementation
    static 
    {
        /** Causes the system to load the CDDisc (.dll, .so, etc) module */
        System.loadLibrary("recls_jni");
    }

/// Members
    private int     m_hSearch;  // The opaque handle to the entry
    private int     m_rc;
}
