/* /////////////////////////////////////////////////////////////////////////////
 * File:        STL_Win32.cpp
 *
 * Purpose:     Implementation file for the STL_Win32 project.
 *
 * Created:     16th August 2003
 * Updated:     24th November 2003
 *
 * Status:      Wizard-generated
 *
 * License:     (Licensed under the Synesis Software Open License)
 *
 *              Copyright (C) 1999-2003, Synesis Software Pty Ltd.
 *              All rights reserved.
 *
 *              www:        http://www.synesis.com.au/software
 *
 *              email:      software@synesis.com.au
 *
 *              This source code is placed into the public domain 2003
 *              by Synesis Software Pty Ltd. There are no restrictions
 *              whatsoever to your use of the software. 
 *
 *              This source code is provided by Synesis Software Pty Ltd "as is"
 *              and any warranties, whether expressed or implied, including, but
 *              not limited to, the implied warranties of merchantability and
 *              fitness for a particular purpose are disclaimed. In no event
 *              shall the Synesis Software Pty Ltd be liable for any direct,
 *              indirect, incidental, special, exemplary, or consequential
 *              damages (including, but not limited to, procurement of
 *              substitute goods or services; loss of use, data, or profits; or
 *              business interruption) however caused and on any theory of
 *              liability, whether in contract, strict liability, or tort
 *              (including negligence or otherwise) arising in any way out of
 *              the use of this software, even if advised of the possibility of
 *              such damage. 
 *
 *              Neither the name of Synesis Software Pty Ltd nor the names of
 *              any subdivisions, employees or agents of Synesis Software Pty
 *              Ltd, nor the names of any other contributors to this software
 *              may be used to endorse or promote products derived from this
 *              software without specific prior written permission. 
 *
 * ////////////////////////////////////////////////////////////////////////// */

// Define this definition to retrict use to platform-independent aspects of the API
//#define RECLS_PURE_API

// Define this to copy file entries into a vector, to exercise the copy semantics
//#define TAKE_COPIES_IN_VECTOR

#include <recls.h>

#if RECLS_VER < RECLS_VER_1_1_1
# error This file now requires version 1.1.1 or later of the recls C API
#endif /* RECLS_VER < 1.1.1 */

#ifdef RECLS_STRICT
# include <stlsoft_nulldef.h>
#endif // RECLS_STRICT

#include <reclstl.h>
#include <reclstl_search_sequence.h>
#include <recls_assert.h>

#ifdef _MSC_VER
# include <crtdbg.h>
#endif /* _MSC_VER */

#include <iostream>
#include <iomanip>

#ifdef TAKE_COPIES_IN_VECTOR
# include <vector>

typedef std::vector<reclstl::search_sequence_a::value_type>     fileentry_t;
#endif /* TAKE_COPIES_IN_VECTOR */

/* /////////////////////////////////////////////////////////////////////////////
 * Namespace
 */

using std::cout;
using std::cerr;
using std::endl;
using recls::uint32_t;
using recls::recls_info_t;
using recls::recls_char_t;
using recls::RECLS_F_FILES;
using recls::RECLS_F_DIRECTORIES;
using recls::RECLS_F_RECURSIVE;
using recls::RECLS_F_DIRECTORY_PARTS;

/* /////////////////////////////////////////////////////////////////////////////
 * Forward declarations
 */

void usage(int bExit);

/* /////////////////////////////////////////////////////////////////////////////
 * Main
 */

/** Program main */
int main(int argc, char *argv[])
{
    int             iRet        =   0;
    int             i;
    char const      *pattern    =   NULL;
    char const      *rootDir    =   NULL;
    uint32_t        flags       =   RECLS_F_RECURSIVE;

    for(i = 1; i < argc; ++i)
    {
        const char  *arg    =   argv[i];

        if(arg[0] == '-')
        {
            if(arg[1] == '-')
            {
                /* -- arguments */
            }
            else
            {
                /* - arguments */
                switch(arg[1])
                {
                    case    'R':    /* Do not recurse */
                        flags &= ~(RECLS_F_RECURSIVE);
                        break;
                    case    'p':    /* Show directory parts */
                        flags |= RECLS_F_DIRECTORY_PARTS;
                        break;
                    case    'f':    /* Find files */
                        flags |= RECLS_F_FILES;
                        break;
                    case    'd':    /* Find directories */
                        flags |= RECLS_F_DIRECTORIES;
                        break;
                    default:
                        usage(1);
                        break;
                }
            }
        }
        else
        {
            /* other arguments */
            if(NULL == pattern)
            {
                pattern = arg;
            }
            else if(NULL == rootDir)
            {
                rootDir = arg;
            }
            else
            {
                usage(1);
            }
        }
    }

    // Search for files if neither files or directories specified
    //
    // Even though this is not necessary, because the recls API provides the
    // same interpretation, it's best to be explicit.
    if(0 == (flags & (RECLS_F_FILES | RECLS_F_DIRECTORIES)))
    {
        flags |= RECLS_F_FILES;
    }

    if(NULL == pattern)
    {
#if defined(RECLS_PLATFORM_IS_WIN32)
        pattern = "*.*";
#else
        pattern = "*";
#endif /* RECLS_PLATFORM_IS_WIN32 */
    }

    if(NULL == rootDir)
    {
        rootDir = ".";
    }

    {
        reclstl::basic_search_sequence<recls_char_t>    search(rootDir, pattern, flags);
#ifdef TAKE_COPIES_IN_VECTOR
        fileentry_t         entries;
#endif /* TAKE_COPIES_IN_VECTOR */

        reclstl::basic_search_sequence<recls_char_t>::const_iterator    begin   =   search.begin();
        reclstl::basic_search_sequence<recls_char_t>::const_iterator    end     =   search.end();

        for(; begin != end; ++begin)
        {
            using reclstl::string_t;

            reclstl::basic_search_sequence<recls_char_t>::value_type    fileEntry   =   *begin;

#ifdef RECLS_PLATFORM_API_WIN32
            recls_char_t            drive       =   fileEntry.get_drive();
#endif /* RECLS_PLATFORM_API_WIN32 */
            string_t                path        =   fileEntry.get_path();
            string_t                dir         =   fileEntry.get_directory();
            string_t                dirPath     =   fileEntry.get_directory_path();
            string_t                file        =   fileEntry.get_file();
            string_t                fileName    =   fileEntry.get_filename();
            string_t                fileExt     =   fileEntry.get_fileext();

            string_t                pathCheck;
            string_t                dirCheck;
            string_t                fileCheck;

#ifdef TAKE_COPIES_IN_VECTOR
            entries.push_back(fileEntry);
#endif /* TAKE_COPIES_IN_VECTOR */

            cout << "  " << path << endl;
            cout << "  " << dirPath << endl;
#ifdef RECLS_PLATFORM_API_WIN32
            if(isupper(path[0]))
            {
                drive = (recls_char_t)toupper(drive);
            }
            else
            {
                drive = (recls_char_t)tolower(drive);
            }
            cout << "  " << drive << endl;
            pathCheck += drive;
            pathCheck += ':';
#endif /* RECLS_PLATFORM_API_WIN32 */
            cout << "    " << dir << endl;

            typedef reclstl::basic_search_sequence<recls_char_t>::value_type::directory_parts_type  directory_parts_type;

            int off = 0;

            if((flags & RECLS_F_DIRECTORY_PARTS) == RECLS_F_DIRECTORY_PARTS)
            {
                directory_parts_type    dirParts    =   fileEntry.get_directory_parts();
                int                     i;

                directory_parts_type::iterator  begin   =   dirParts.begin();
                directory_parts_type::iterator  end     =   dirParts.end();

                for(i = 0; begin != end; ++begin, ++i)
                {
                    string_t    dirPart =   *begin;

                    off += dirPart.length();
                    cout << "    " << std::setw(off) << dirPart << std::setw(1) << endl;

                    pathCheck += dirPart;
                    dirCheck += dirPart;
                }
            }
            else
            {
                off += dir.length();
                pathCheck += dir;
            }

            off += file.length();
            cout << "    " << std::setw(off) << file << std::setw(1) << endl;
            if(fileExt.length() > 0)
            {
                cout << "    " << std::setw(off - (1 + fileExt.length())) << fileName << std::setw(1) << endl;
                cout << "    " << std::setw(off) << fileExt << std::setw(1) << endl;
            }
            else
            {
                cout << "    " << std::setw(off) << fileName << std::setw(1) << endl;
            }

            fileCheck = fileName;
            if(0 < fileExt.length())
            {
                fileCheck += '.';
                fileCheck += fileExt;
            }

            pathCheck += file;

            // Now validate the components
#ifdef RECLS_PLATFORM_API_WIN32
            recls_assert(path == pathCheck);
            if(path != pathCheck)
#else /* ? RECLS_PLATFORM_API_WIN32 */
            recls_assert(0 == strcmp(path.c_str() + 2, pathCheck.c_str()));
            if(0 != strcmp(path.c_str() + 2, pathCheck.c_str()))
#endif /* RECLS_PLATFORM_API_WIN32 */
            {
                cerr << "Path is different from path components" << endl << "\tpath:  " << path << "" << endl << "\tparts: " << pathCheck << endl << endl;

                abort();
            }
#ifdef RECLS_PLATFORM_API_WIN32
            recls_assert(dirPath[0] == drive && dir == dirPath.c_str() + 2);
            if( dirPath[0] != drive ||
                dir != dirPath.c_str() + 2)
#else
            recls_assert(dirPath == dir);
            if(dirPath != dir)
#endif /* RECLS_PLATFORM_API_WIN32 */
            {
                cerr << "DirectoryPath is different from Directory" << endl << "\tdirPath:  " << "%s" << "" << endl << "\tdir: " << "%s" << endl << endl;

                abort();
            }
            if((flags & RECLS_F_DIRECTORY_PARTS) == RECLS_F_DIRECTORY_PARTS)
            {
                recls_assert(dir == dirCheck);
                if(dir != dirCheck)
                {
                    cerr << "Directory is different from directory components" << endl << "\tpath:  " << dir << "" << endl << "\tparts: " << dirCheck << endl << endl;

                    abort();
                }
            }
            recls_assert(file == fileCheck);
            if(file != fileCheck)
            {
                cerr << "File is different from file components" << endl << "\tpath:  " << "%s" << "" << endl << "\tparts: " << "%s" << endl << endl;

                abort();
            }

            if(fileEntry.is_readonly())
            {
                cout << "    - Read-only" << endl;
            }
            if(fileEntry.is_directory())
            {
                cout << "    - Directory" << endl;
            }
            else
            {
                cout << "    - File" << endl;
            }
            if(fileEntry.is_link())
            {
                cout << "    - Link" << endl;
            }
        }

#ifdef TAKE_COPIES_IN_VECTOR
        entries.clear();
#endif /* TAKE_COPIES_IN_VECTOR */
    }

#ifdef _MSC_VER
    _CrtDumpMemoryLeaks();
#endif /* _MSC_VER */

    return iRet;
}

/* /////////////////////////////////////////////////////////////////////////////
 * Function implementations
 */

/** Displays program usage */
void usage(int bExit)
{
    cout << "recls C++ Test Program: STL_Win32" << endl << endl;
    cout << "Usage: STL_Win32 [-R] [<pattern>] [<root-dir>]" << endl;
    cout << "\t-R               -   does not recurse; recursive search is the default" << endl;
    cout << "\t<pattern>        -   search pattern, e.g. \"*.cpp\"; default is to search for all files" << endl;
    cout << "\t<root-dir>       -   root directory of search; default is current working directory" << endl;

    if(bExit)
    {
        exit(1);
    }
}

/* //////////////////////////////////////////////////////////////////////////
 * End of file
 */
